## generalized pooling problem (network design)
## Leo Liberti (from Greenberg & Floudas)

# supplementary pipe activation variables
var zIn{I,P} binary, >= 0, <= 1;
var zOut{P,J} binary, >= 0, <= 1;

## objective function
minimize cost: 
  sum{i in I} cI[i]*sum{p in P : i in Nm[p]} x[i,p]
  - sum{j in J} cJ[j]*sum{p in P : j in Np[p]} y[p,j] 
  + sum{i in I, p in P : i in Nm[p]} zIn[i,p]
  + sum{p in P, j in J : j in Np[p]} zOut[p,j];

## constraints

# pipe activation
subject to activeIn{i in I, p in P : i in Nm[p]}: x[i,p] <= S[i]*zIn[i,p];
subject to activeOut{j in J, p in P : j in Np[p]}: y[p,j] <= D[j]*zOut[p,j];

# do not exceed supply
subject to supply{i in I}: sum{p in P : i in Nm[p]} x[i,p] <= S[i];

# do not exceed demand (rather than satisfy demand)
subject to demand{j in J}: sum{p in P : j in Np[p]} y[p,j] <= D[j];

# mass flow across pools
subject to flow_balance{p in P}:
  sum{i in Nm[p]} x[i,p] = sum{j in Np[p]} y[p,j];

# attribute balance across pools
subject to attr_balance{p in P, k in K}: 
  sum{i in Nm[p]} A[i,k]*x[i,p] = sum{i in Nm[p]} q[p,k]*x[i,p];

# bounds on output attributes (bounds are on L<=Q<=U) 
subject to attr_product{j in J, k in K}:
  sum{p in P : j in Np[p]} q[p,k]*y[p,j] =
    sum{p in P : j in Np[p]} Q[j,k]*y[p,j];

